//
//  AppDelegate.m
//  SDKReferenceApp
//
//  Created by Michael Murphy on 25/11/2014.
//  Copyright (c) 2014 Proxama. All rights reserved.
//

#import "AppDelegate.h"

#import <TapPointSDK/TapPointSDK.h>

#import <CoreLocation/CoreLocation.h>

@interface AppDelegate ()

/**
 * Instance of the location manager, used for requesting permission to use location services.
 */
@property (nonatomic, strong) CLLocationManager *locationManager;

/**
 * This is a callback that is used for responding to trigger detected notifications. The TapPoint SDK
 * will send a NSNotification named as PRXTriggersDetectedNotification that will contain a PRXTrigger.
 *
 * The PRXTrigger that has been detected can then be used to send a report describing that the user has been
 * notified of a trigger event.
 *
 * @param notification NSNotification that contains the detected PRXTrigger.
 */
- (void)triggerDetected:(NSNotification *)notification;

/**
 * Helper method to display an alert to the user.
 *
 * @param error The error returned from the TapPoint SDK.
 */
- (void)displayAlertForError:(NSError *)error;

@end

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
	_locationManager = [[CLLocationManager alloc] init];
	if ([self.locationManager respondsToSelector:@selector(requestAlwaysAuthorization)]) {
		if ([CLLocationManager authorizationStatus] == kCLAuthorizationStatusNotDetermined) {
			[self.locationManager requestAlwaysAuthorization];
		}
	}

	[[PRXAuthentication sharedAuthenticationManager] authenticateWithAppName:@"sdk_reference_app" success:^{
		NSLog(@"TapPoint: Successfully authenticated with TapPoint.");
		
		[[PRXSynchronisation sharedSynchronisationManager] synchroniseWithSuccess:^(PRXSyncResult *result) {
			NSLog(@"TapPoint: Synchronised with TapPoint.");
			
			NSLog(@"%lu triggers were added", (unsigned long)result.triggersAdded.count);
			NSLog(@"%lu triggers were removed", (unsigned long)result.triggersRemoved.count);

			for (PRXTrigger *trigger in result.triggersAdded) {
				NSLog(@"Added Trigger ID: %@ with payload: %@", trigger.identifier, trigger.payload);
			}
			
			[[NSNotificationCenter defaultCenter] addObserver:self
													 selector:@selector(triggerDetected:)
														 name:PRXTriggersDetectedNotification
													   object:nil];

			[[PRXTriggers sharedTriggersManager] startMonitoring];
		} failure:^(NSError *error) {
			NSLog(@"TapPoint: Unable to synchronise with TapPoint.");
			[self displayAlertForError:error];
		}];
		
	} failure:^(NSError *error) {
		NSLog(@"TapPoint: Unable to authenticate with TapPoint.");
		[self displayAlertForError:error];
	}];

	[[UIApplication sharedApplication] setMinimumBackgroundFetchInterval:UIApplicationBackgroundFetchIntervalMinimum];
	
	return YES;
}

- (void)triggerDetected:(NSNotification *)notification
{
	NSLog(@"TapPoint Trigger Detected!");
	
	/**
	 * Below shows how to report on a trigger event that has occurred for both when the application is in the background
	 * and the foreground. Events that are reported on elsewhere (i.e. in a view controller), will only be for events 
	 * that occur whilst the application is in the foreground.
	 */
	PRXTrigger *trigger = notification.userInfo[PRXDetectedTriggerNotificationKeyForTrigger];
	[[PRXReporting sharedReportingManager] reportTriggerEvent:PRXTriggerEventTypeUserNotified
													triggerID:trigger.identifier];
	
	NSLog(@"Reporting trigger event: UserNotified: Trigger ID: %@", trigger.identifier);
	NSLog(@"Trigger Payload: %@", trigger.payload);
}

- (void)displayAlertForError:(NSError *)error
{
	NSLog(@"TapPoint: Error code %ld", (long) error.code);
	NSLog(@"TapPoint: %@ - %@", error.localizedFailureReason, error.localizedRecoverySuggestion);

	dispatch_async(dispatch_get_main_queue(), ^{
		[[[UIAlertView alloc] initWithTitle:error.localizedDescription
									message:error.localizedFailureReason
								   delegate:self
						  cancelButtonTitle:@"Done"
						  otherButtonTitles:nil] show];
	});
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

-(void)application:(UIApplication *)application performFetchWithCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler
{
	[[PRXSynchronisation sharedSynchronisationManager] synchroniseWithSuccess:^(PRXSyncResult *result) {
		if (completionHandler) {
			if (result.triggersAdded.count > 0 || result.triggersRemoved.count > 0) {
				completionHandler(UIBackgroundFetchResultNewData);
			} else {
				completionHandler(UIBackgroundFetchResultNoData);
			}
		}
	} failure:^(NSError *error) {
		if (completionHandler) {
			completionHandler(UIBackgroundFetchResultFailed);
		}
	}];
}

@end
