//
//  BeaconEventsTableViewController.m
//  SDKReferenceApp
//
//  Created by Michael Murphy on 19/11/2014.
//  Copyright (c) 2014 Proxama. All rights reserved.
//

#import "BeaconEventsTableViewController.h"

#import "BeaconEventDetailViewController.h"
#import "BeaconEventTableViewCell.h"

#import <TapPointSDK/TapPointSDK.h>

@interface BeaconEventsTableViewController ()

/**
 * A mutable array of dictionaries. Each dictionary contains information about a single trigger firing event.
 */
@property (strong, nonatomic) NSMutableArray *beaconEventInfoObjects;

/**
 * The label shown if there are no rows in the table.
 */
@property (weak, nonatomic) IBOutlet UILabel *noBeaconsTriggeredLabel;

/**
 * The main table view managed by this view controller.
 */
@property (weak, nonatomic) IBOutlet UITableView *tableView;

/**
 * Date formatter object that formats the time for display in the table view cell.
 */
@property (strong, nonatomic) NSDateFormatter *dateFormatter;

/**
 * Callback handler for processing trigger notifications.
 *
 * @param notification NSNotification posted by the default notification center.
 */
- (void)triggerDetected:(NSNotification *)notification;

@end

@implementation BeaconEventsTableViewController

- (void)viewDidLoad {
	[super viewDidLoad];

	UINib *beaconEventTableViewCellNib = [UINib nibWithNibName:@"BeaconEventTableViewCell" bundle:nil];
	[self.tableView registerNib:beaconEventTableViewCellNib forCellReuseIdentifier:@"BeaconEventCell"];
	
	// Start monitoring for beacon events.
	// This might happen before the application is registered, if this is the first launch
	// and the call from the AppDelegate to authenticate with TapPoint has not yet completed.
	// If that happens, it will log: [PRXTriggers] [startMonitoring] Application has not been
	// authenticated. Authenticate before calling this method.
	[[PRXTriggers sharedTriggersManager] startMonitoring];
	
	// If we were already registered for notifications, unsubscribe first.
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(triggerDetected:)
												 name:PRXTriggersDetectedNotification
											   object:nil];
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (NSDateFormatter *)dateFormatter
{
	if (_dateFormatter == nil) {
		_dateFormatter = [[NSDateFormatter alloc] init];
		[_dateFormatter setDateStyle:NSDateFormatterNoStyle];
		[_dateFormatter setTimeStyle:NSDateFormatterMediumStyle];
	}
	
	return _dateFormatter;
}

- (NSMutableArray *)beaconEventInfoObjects
{
	if (_beaconEventInfoObjects == nil) {
		_beaconEventInfoObjects = [[NSMutableArray alloc] init];
	}
	
	return _beaconEventInfoObjects;
}

- (void)insertNewBeaconEventInfoObject:(id)object {
	[self.beaconEventInfoObjects insertObject:object atIndex:0];
	NSIndexPath *indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
	[self.tableView insertRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
}

- (void)triggerDetected:(NSNotification *)notification
{
	// Notification's user info will contain both the custom payload, and the trigger ID
	NSLog(@"TapPoint: Trigger fired with user info %@", [[notification userInfo] description]);
	
	[self insertNewBeaconEventInfoObject:@{ @"date" : [NSDate date], @"notification" : notification }];
}

#pragma mark - Segues

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
	if ([[segue identifier] isEqualToString:@"showDetail"]) {
		NSIndexPath *indexPath = [self.tableView indexPathForSelectedRow];
		NSDictionary *object = self.beaconEventInfoObjects[indexPath.row];
		[[segue destinationViewController] setDetailItem:object];
	}
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	/**
	 * We retrieve the trigger event object that we previously received an event for
	 * at the given index path.
	 */
	NSDictionary *object = self.beaconEventInfoObjects[indexPath.row];
	NSNotification *notification = object[@"notification"];
	
	// We now extract the PRXTrigger from the notification we received previously.
	PRXTrigger *trigger = notification.userInfo[PRXDetectedTriggerNotificationKeyForTrigger];
	
	/**
	 * As the user is performing an interaction here with the trigger event, we record that this
	 * interaction has occurred. This could be via any form of UI element where the user is
	 * is interacting with the trigger event and wish to find out more about this event.
	 */
	[[PRXReporting sharedReportingManager] reportTriggerEvent:PRXTriggerEventTypeUserSelected triggerID:trigger.identifier];
	
	NSLog(@"Reporting trigger event: UserSelected: Trigger ID: %@", trigger.identifier);
	
	[self.tableView deselectRowAtIndexPath:indexPath animated:YES];
	[self.tableView reloadRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
}

#pragma mark - Table View

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	NSInteger result = [self.beaconEventInfoObjects count];
	if (result > 0) {
		self.noBeaconsTriggeredLabel.hidden = YES;
		self.tableView.hidden = NO;
	}
	else {
		self.noBeaconsTriggeredLabel.hidden = NO;
		self.tableView.hidden = YES;
	}
	return result;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	BeaconEventTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"BeaconEventCell" forIndexPath:indexPath];

	NSDictionary *object = self.beaconEventInfoObjects[indexPath.row];
	
	NSNotification *notification = object[@"notification"];
	PRXTrigger *trigger = notification.userInfo[PRXDetectedTriggerNotificationKeyForTrigger];
	
	NSDictionary *payloadData = trigger.payload[@"data"];
	
	cell.titleLabel.text = payloadData[@"title"] ?: notification.name;
	
	cell.subtitleLabel.text = payloadData[@"subtitle"] ?: @"No subtitle";
	
	cell.timeLabel.text = [self.dateFormatter stringFromDate:object[@"date"]];

	NSString *imageName = payloadData[@"imageName"] ?: nil;
	cell.iconImageView.image = [UIImage imageNamed:imageName];
	
	return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
	return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
	if (editingStyle == UITableViewCellEditingStyleDelete) {
		[self.beaconEventInfoObjects removeObjectAtIndex:indexPath.row];
		[tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
	}
}

@end
